<?php
/*
Simple:Press
Blog Linking - Blog side comment support
$LastChangedDate: 2015-06-08 07:32:52 -0700 (Mon, 08 Jun 2015) $
$Rev: 12989 $
*/

if (preg_match('#'.basename(__FILE__).'#', $_SERVER['PHP_SELF'])) die('Access denied - you cannot directly call this file');

# ------------------------------------------------------------------
# sp_process_new_comment()
#
# NOT DIRECTLY CALLABLE - 'comment_post' & 'wp_set_comment_status'
# Fires when a new comment is added (Action hook)
#	$cid:		ID of new comment
#	$status:	"spam", "1' or "0" ("1" is approved)
#	-- OR --	"approve", "delete", "spam", "hold"
# ------------------------------------------------------------------
function sp_process_new_comment($cid, $commentstatus) {
	$commentstatus = trim($commentstatus);
	if ($commentstatus == '0' || $commentstatus == 'spam' || $commentstatus == 'hold') return;

	# First - if not approved then back out
	if ($commentstatus == '1' || $commentstatus == "approve") {
		# Next find the post ID to see if it is a linked post
		$crecord = spdb_table(SFWPCOMMENTS, "comment_ID=$cid", 'row');
		if (!$crecord) return;
		# if a pingback or trackback then leave it alone
		if ($crecord->comment_type == 'pingback' || $crecord->comment_type == 'trackback') return;
		# Is it a linked post?
		$links = sp_blog_links_control('read', $crecord->comment_post_ID);
		if (!$links) return;

		# This means it is an approved comment with linking
		sp_create_post_from_comment($crecord, $links);

		# Do we delete original comment?
		$sfpostlinking = sp_get_option('sfpostlinking');
		if ($sfpostlinking['sfkillcomment']) spdb_query("DELETE FROM ".SFWPCOMMENTS." WHERE comment_ID=".$cid);
	}

	# And just in case - was it a delete action?
	if ($commentstatus == 'delete') {
		# Check posts for thie comment ID
		$post = spdb_table(SFPOSTS, "comment_ID=$cid", 'row');
		if ($post) {
			$sfpostlinking = sp_get_option('sfpostlinking');
			if ($sfpostlinking['sfeditcomment']) sp_delete_post($post->post_id, $post->topic_id, $post->forum_id, false);
		}
	}
}

# ------------------------------------------------------------------
# sp_create_post_from_comment()
#
# Create new topic post from comment
#	$crecord:		The comment record
#	$links:			Blog Linking IDs
# ------------------------------------------------------------------
function sp_create_post_from_comment($crecord, $links) {
	global $spVars;

	include_once SF_PLUGIN_DIR.'/forum/library/sp-post-support.php';
	include_once SF_PLUGIN_DIR.'/forum/database/sp-db-newposts.php';
	include_once SF_PLUGIN_DIR.'/forum/database/sp-db-management.php';

    sp_forum_api_support();

	$p = new spPost;
	$p->action = 'post';

    if (empty($crecord->user_id)) {
    	$p->userid = 0;
    	$p->admin = false;
    	$p->moderator = false;
    	$p->member = false;
    	$p->guest = true;
    	$p->newpost['guestname'] = sp_filter_name_save($crecord->comment_author);
        $p->newpost['guestemail'] = sp_filter_email_save($crecord->comment_author_email);
    } else {
        $user = sp_get_user($crecord->user_id);
    	$p->userid	= $user->ID;
    	$p->admin = $user->admin;
    	$p->moderator = $user->moderator;
    	$p->member = $user->member;
    	$p->guest = $user->guest;
    	$p->newpost['postername'] = $user->display_name;
    	$p->newpost['posteremail'] 	= $user->user_email;
    }

	$topicid = sp_esc_int($links->topic_id);
	$forumid = sp_esc_int($links->forum_id);

	$p->newpost['forumid'] = $forumid;
	$p->newpost['topicid'] = $topicid;
	$p->newpost['forumslug'] = spdb_table(SFFORUMS, "forum_id=$forumid", 'forum_slug');
	$p->newpost['topicslug'] = spdb_table(SFTOPICS, "topic_id=$topicid", 'topic_slug');

	$p->validatePermission();
	if (!$p->abort) {
    	$p->newpost['topicname'] = spdb_table(SFTOPICS, "topic_id=$topicid", 'topic_name');
    	$p->newpost['postcontent'] = $crecord->comment_content;
    	$p->newpost['userid'] = $p->userid;
    	$p->newpost['ip'] = $crecord->comment_author_ip;
		$p->validateData();
		if (!$p->abort) {
			$p->saveData();
			if ($p->abort) {
				trigger_error('Linking - Saving'.': '.$p->message, E_USER_WARNING);
			}

            # post saved - add in the wp comment id
			$sql = "UPDATE ".SFPOSTS." SET comment_id=$crecord->comment_ID WHERE post_id=".$p->newpost['postid'];
			spdb_query($sql);
		} else {
			trigger_error('Linking - Validation'.': '.$p->message, E_USER_WARNING);
		}
	} else {
		trigger_error('Linking - Permission'.': '.$p->message, E_USER_WARNING);
	}
}

# ------------------------------------------------------------------
# sp_update_comment_post()
#
# NOT DIRECTLY CALLABLE
# Updates topic post when comment is edited
#	$cid	Comment ID
# ------------------------------------------------------------------
function sp_update_comment_post($cid) {
	# find the post ID to see if it is a linked post
	$crecord = spdb_table(SFWPCOMMENTS, "comment_ID=$cid", 'row');
	if (!$crecord) return;
	$links = sp_blog_links_control('read', $crecord->comment_post_ID);
	if (!$links) return;

	# So we need to perform an update
	$sfpostlinking = sp_get_option('sfpostlinking');
	if ($sfpostlinking['sfeditcomment']) {
		$postcontent = $crecord->comment_content;
		if ($postcontent) {
			$postid = spdb_table(SFPOSTS, "comment_id=$cid", 'post_id');
			if ($postid) {
				$postcontent = sp_filter_content_save($postcontent, 'edit', true, SFPOSTS, 'post_content');
				$sql = "UPDATE ".SFPOSTS." SET post_content='".$postcontent."' WHERE post_id=".$postid;
				spdb_query($sql);
			}
		}
	}
}

# ------------------------------------------------------------------
# sp_topic_as_comments()
#
# NOT DIRECTLY CALLABLE
# Adds the topic posts to the comments stream
#	$comments	Passed in by the comments_array filter
# ------------------------------------------------------------------
function sp_topic_as_comments($comments) {
	global $wp_query, $spThisUser;

	$sfpostlinking = sp_get_option('sfpostlinking');

	if ($comments) {
		$postid = $comments[0]->comment_post_ID;
	} else {
		$postid = $wp_query->post->ID;
	}
	$links = sp_blog_links_control('read', $postid);
	if (!$links) return $comments;

	sp_forum_api_support();

	$topicid = $links->topic_id;

	$thread = sp_get_thread_for_comments($topicid, $sfpostlinking['sfhideduplicate']);
	if ($thread) {
		$index = count($comments);

		foreach ($thread as $post) {
        	# quick permission check
            if (!sp_can_view($links->forum_id, 'post-content', $spThisUser->ID, $post['user_id'], $post['topic_id'], $post['post_id'])) continue;

            $comments[$index] = new stdClass();
			$comments[$index]->comment_ID = $links->forum_id.'@'.$links->topic_id.'@'.$post['post_id'];
			$comments[$index]->comment_post_ID = $postid;

			if (!$post['user_id']) {
				$comments[$index]->comment_author = sp_filter_name_display($post['guest_name']);
				$comments[$index]->comment_author_email = sp_filter_email_display($post['guest_email']);
				$comments[$index]->comment_author_url = "";
			} else {
				$comments[$index]->comment_author = sp_filter_name_display($post['display_name']);
				$comments[$index]->comment_author_email = sp_filter_email_display($post['user_email']);
				$comments[$index]->comment_author_url = sp_check_url($post['user_url']);
			}
			$comments[$index]->comment_author_IP = $post['poster_ip'];
			$comments[$index]->comment_date = $post['post_date'];
			$comments[$index]->comment_date_gmt = $post['post_date'];

			if ($post['post_status'] != 0) {
				$comments[$index]->comment_content = '<b><em>'.__("Post Awaiting Approval by Forum Administrator", 'sp-linking').'</em></b>';
			} else {
				$comments[$index]->comment_content = sp_filter_content_display($post['post_content']);
			}

			$comments[$index]->comment_karma = 0;
			$comments[$index]->comment_approved = 1;
			$comments[$index]->comment_agent = "";

			if ($sfpostlinking['sflinkcomments'] == 2) {
				$comments[$index]->comment_type = "comment";
			} else {
				$comments[$index]->comment_type = "forum";
			}
			$comments[$index]->comment_parent = 0;
			$comments[$index]->user_id = $post['user_id'];
			$comments[$index]->comment_subscribe = "N";

			$index++;
		}
	}
	if ($sfpostlinking['sflinkcomments'] == 2) usort($comments, 'sp_sort_comments');

	return $comments;
}

function sp_sort_comments($a, $b) {
    if ($a->comment_date == $b->comment_date) return 0;

    $sort=get_option('comment_order');
    if ($sort == 'asc') {
	    return ($a->comment_date < $b->comment_date) ? -1 : 1;
	} else {
	    return ($a->comment_date > $b->comment_date) ? 1 : -1;
	}
}

function sp_get_thread_for_comments($topicid, $hidedupes) {
	$hide = '';
	if ($hidedupes) $hide = " AND comment_id IS NULL ";

	$records = spdb_select('set',
			"SELECT ".SFPOSTS.".post_id, ".SFPOSTS.".topic_id, post_content, post_date, ".SFPOSTS.".user_id, guest_name, guest_email, post_status, poster_ip,
			".SFMEMBERS.".display_name, user_url, user_email
			 FROM ".SFPOSTS."
			 LEFT JOIN ".SFUSERS." ON ".SFPOSTS.".user_id = ".SFUSERS.".ID
			 LEFT JOIN ".SFMEMBERS." ON ".SFPOSTS.".user_id = ".SFMEMBERS.".user_id
			 WHERE topic_id = ".$topicid." AND post_index > 1".$hide."
			 ORDER BY post_id ".strtoupper(get_option('comment_order')).";", ARRAY_A);

	return $records;
}

# ------------------------------------------------------------------
# sp_remove_edit_comment_link()
#
# NOT DIRECTLY CALLABLE
# Sets the forum post to the 'edit' link in cmments when viewed
# bya  site admin.
# ------------------------------------------------------------------
function sp_remove_edit_comment_link($link, $id) {
	if (strpos($id, '@')) {
		$link = '';
		$target = explode('@', $id);

		if ((isset($target[0]) && $target[0] != 0) && (isset($target[1]) && $target[1] != 0) && (isset($target[2]) && $target[2] != 0)) {
			$link = '<span><form action="'.sp_build_url(spdb_table(SFFORUMS, 'forum_id='.$target[0], 'forum_slug'), spdb_table(SFTOPICS, 'topic_id='.$target[1], 'topic_slug'), 0, $target[2]).'" method="post" name="admineditpost'.$target[2].'">'."\n";
			$link.= '<input type="hidden" name="adminedit" value="'.$target[2].'" />'."\n";
			$link.= '<a class="comment-edit-link" href="javascript:document.admineditpost'.$target[2].'.submit();">('.__("Edit", 'sp-linking').')</a>'."\n";
			$link.= '</form></span>'."\n";
		}
	}
	return $link;
}

# ------------------------------------------------------------------
# sp_add_comment_type()
#
# NOT DIRECTLY CALLABLE
# Sets up new comment typoe of 'forum; so topic posts as comments
# can show users avatar
# 	$list	Current list of commnent types
# ------------------------------------------------------------------
function sp_add_comment_type($list) {
	$list[] = 'forum';
	return $list;
}
?>